/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / C O R E / D B 2 / D I A G L O G . J A V A      ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Wrapper class, implementing db2diag.log file          ## */
/* ##               related stuff.                                        ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-08-09                                            ## */
/* ##  Changes ...: ----- DiagLog.java V00.99/R020 ---------------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- DiagLog.java V00.99/R011 ---------------------- ## */
/* ##               2002-10-23 - Method toInternalReturnCode() and        ## */
/* ##                            toSqlCode() throughs the                 ## */
/* ##                            NullPointerException exception too.      ## */
/* ##               2002-10-23 - Added private constructor to prevent     ## */
/* ##                            the instantiation of this call.          ## */
/* ##               2002-10-20 - The method toSqlCode() now make use      ## */
/* ##                            of FFFF_TOKEN as well.                   ## */
/* ##               2002-10-09 - Renamed the class and package (was       ## */
/* ##                            org.edelbyte.DbtuTools.DB2DiagLog).      ## */
/* ##               2002-09-01 - Method toInternalReturnCode() and        ## */
/* ##                            toSqlCode() throughs another             ## */
/* ##                            exception now (was NumberFormatExcept.)  ## */
/* ##               ----- DiagLog.java V00.99/R010 ---------------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
package org.dbtutools.DbtuTools.core.db2;
/* ------------------------------------------------------------------------- */





/* ##### Class definition */
/** ************************************************************************
 * Wrapper class, implementing db2diag.log file related stuff.
 * It cannot be instantiated. 
 * @author Daniel Scheibli &lt;daniel.scheibli at edelbyte.org&gt;
 * ************************************************************************* */
public class DiagLog
{
 /* ##### Presettings (class fields)
 /* ------------------------------------------------------------------------ */
 /** ***********************************************************************
  * Constant holding a special token for the hexadecimal code,
  * found within the <code>db2diag.log</code> file.
  * ************************************************************************ */
 public static final java.lang.String FFFF_TOKEN = "FFFF";
 /* ------------------------------------------------------------------------ */





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   D I A G L O G  ()                                                ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Private declared Constructor, preventing the instantiation
  * of the <code>DiagLog</code> class.
  * ************************************************************************ */
 private DiagLog( )
 { }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   T O I N T E R N A L R E T U R N C O D E ()                       ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Converts a given hexadecimal code found in the <code>db2diag.log</code>
  * file into an DB2 Internal Return Code.<br>
  * <br>
  * <i>Remark:</i> Please notice; the hexadecimal code could
  * represent a SQL Message Code as well. For more information
  * about how to interpret the <code>db2diag.log</code>, please have
  * a look into the <i>DB2 Documentation - Troubleshooting Guide</i>.
  * @see #toSqlCode(java.lang.String, java.lang.String)
  *
  * @param oParamTokenA
  *        The String holding the first (4 digits long) token.
  * @param oParamTokenB
  *        The String holding the second (4 digits long) token.
  * @return The calculated DB2 Internal Return Code (in upper case).
  * @exception java.lang.NumberFormatException
  *            For the case, that the given tokens has an invalid
  *            format or contains unexpected characters. If none of
  *            the tokens match {@link #FFFF_TOKEN}, the exception
  *            is thrown as well.
  * @exception java.lang.NullPointerException
  *            If the <code>oParamTokenA</code> or
  *            <code>oParamTokenB</code> argument
  *            was <code>null</null>.      
  * ************************************************************************ */
 public static java.lang.String toInternalReturnCode( java.lang.String oParamTokenA,
                                                      java.lang.String oParamTokenB ) throws java.lang.NumberFormatException,
 						                                             java.lang.NullPointerException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.lang.String oTokenA       = oParamTokenA.toUpperCase();
  java.lang.String oTokenB       = oParamTokenB.toUpperCase();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int              iFfffTokenPos = 0;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  char[]           acDummy;
  char             cDummy;
  int              iDummy;
  /* ----------------------------------------------------------------------- */  



  /* ##### Check the given parameters */
  /* ----------------------------------------------------------------------- */
  if( (oTokenA == null) ||
      (oTokenB == null) )
  { throw new java.lang.NullPointerException(); }  
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( (oTokenA.length() != 4) ||
      (oTokenB.length() != 4) ) 
  { throw new java.lang.NumberFormatException( "Each token must have 4 characters." ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  try
  {
   iDummy = java.lang.Integer.parseInt(oTokenA, 16);
   iDummy = java.lang.Integer.parseInt(oTokenB, 16);   
  }
  catch( java.lang.NumberFormatException oException )
  { throw new java.lang.NumberFormatException( "Each token must be a valid hex value." ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oTokenA.compareTo( FFFF_TOKEN ) == 0 )
  { iFfffTokenPos += 1; }  
  if( oTokenB.compareTo( FFFF_TOKEN ) == 0 )
  { iFfffTokenPos += 2; }  
  /* ----------------------------------------------------------------------- */



  /* ##### Return result - based on position of the 'FFFF' token */
  /* ----------------------------------------------------------------------- */
  switch( iFfffTokenPos )
  {
   case 0:  /* ##### If no 'FFFF' token was found,... */
        throw new java.lang.NumberFormatException( "At least one token must be 'FFFF'." );
   case 1:  /* ##### If first token is 'FFFF' token,... */
        return( oTokenB );
   case 2:  /* ##### If second token is 'FFFF' token,... */
        acDummy = oTokenA.toCharArray();
	cDummy  = acDummy[0]; acDummy[0] = acDummy[2]; acDummy[2] = cDummy;
	cDummy  = acDummy[1]; acDummy[1] = acDummy[3]; acDummy[3] = cDummy;
        return( new java.lang.String(acDummy) );	
   case 3:  /* ##### If both tokens are 'FFFF' tokens,... */
        return( FFFF_TOKEN );
   default: /* ##### Unexpected error,... */
        throw new java.lang.NumberFormatException( "Unexpected error." );
  }
  /* ----------------------------------------------------------------------- */
 }			    





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   T O S Q L C O D E ()                                             ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Converts a given hexadecimal code found in the <code>db2diag.log</code>
  * file into an SQL Message Code.<br>
  * <br>
  * <i>Remark:</i> Please notice; the hexadecimal code could
  * represent a DB2 Internal Return Code as well. For more information
  * about how to interpret the <code>db2diag.log</code>, please have
  * a look into the <i>DB2 Documentation - Troubleshooting Guide</i>.
  * @see #toInternalReturnCode(java.lang.String, java.lang.String)
  *
  * @param oParamTokenA
  *        The String holding the first (4 digits long) token.
  * @param oParamTokenB
  *        The String holding the second (4 digits long) token.
  * @return The calculated SQL Message Code.
  * @exception java.lang.NumberFormatException
  *            For the case, that the given tokens has an invalid
  *            format or contains unexpected characters. If none of
  *            the tokens match {@link #FFFF_TOKEN}, the exception
  *            is thrown as well.
  * @exception java.lang.NullPointerException
  *            If the <code>oParamTokenA</code> or
  *            <code>oParamTokenB</code> argument
  *            was <code>null</null>.
  * ************************************************************************ */
 public static int toSqlCode( java.lang.String oParamTokenA,
                              java.lang.String oParamTokenB ) throws java.lang.NumberFormatException,
 						                     java.lang.NullPointerException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.lang.String oErrorToken;
  int              iDummy;
  /* ----------------------------------------------------------------------- */  


 
  /* ##### Check the given parameters                           */
  /* ##### (by converting to the 'Internal Return Code' of DB2; */
  /* ##### the called method still does all the checking!)      */
  /* ----------------------------------------------------------------------- */
  oErrorToken = org.dbtutools.DbtuTools.core.db2.DiagLog.toInternalReturnCode(oParamTokenA, oParamTokenB);
  /* ----------------------------------------------------------------------- */  
 
 
 
  /* ##### Compute result and return its value */
  /* ----------------------------------------------------------------------- */  
  try
  {
   iDummy  = java.lang.Integer.parseInt(oErrorToken, 16);
   iDummy -= java.lang.Integer.parseInt(FFFF_TOKEN, 16);
   iDummy -= 1;
  }
  catch( java.lang.NumberFormatException oException )
  { throw new java.lang.NumberFormatException( "Unexpected error." ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  return( iDummy );
  /* ----------------------------------------------------------------------- */   
 }
}
/* ------------------------------------------------------------------------- */




